from .subginery import Subginery, _SubgineriesFunit
from ..engineries import climate


class Climate(Subginery):
    def on_level_changed(self, eng):
        if isinstance(eng, climate.TemperatureSensor):
            funit = Temperature
            value = 0
            n = 0
            for current_eng in self.engineries:
                if isinstance(current_eng, climate.TemperatureSensor):
                    if current_eng.values.get(funit):
                        value += current_eng.values[funit]
                        n += 1
            if n == 0:
                value = None
            else:
                value = int(value / n)
        else:
            return

        self.publish_state(funit, value)


class _ClimatingFunit(_SubgineriesFunit):
    entity_type = Climate
    target_class = climate._Climate


Climate.funit_class = _ClimatingFunit


class Temperature(_ClimatingFunit):
    target_class = climate.TemperatureSensor
    target_command = climate.Temperature


class TargetTemperature(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.TargetTemperature


class ThermoregulatorsOn(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.On


class Preset(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.Preset


class Mode(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.Mode


class FanSpeedMode(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.FanSpeedMode


class LouverMode(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.LouverMode


class FansOn(_ClimatingFunit):
    target_class = climate.Fan
    target_command = climate.On


class HeatedFloorsOn(_ClimatingFunit):
    target_class = climate.HeatedFloor
    target_command = climate.On


Climate.settable_units = [
    TargetTemperature,
    ThermoregulatorsOn,
    Preset,
    Mode,
    FanSpeedMode,
    LouverMode,
    FansOn,
    HeatedFloorsOn
]
