let chartData = {
    labels: [],
    periods: null,
};

function requestChartData(id, topic, from, to, place) {
    /* Запрашивает данные потребления как по локации,
    так и по отдельным потребителям в световой зоне
     */
    if (!['location', 'area'].includes(place)) {
        throw 'Неверное значение параметра place: ' + place + '. Укажите "location" или "area"';

    }
    chartData.periods = chartData.periods ?? calculatePeriods(id, from, to);
    const algo = "integral";

    chartData.periods.forEach(function(period, i) {
        client.send(topic,
            JSON.stringify({
                    value: {
                        algorithm: algo,
                        startTime: period.from,
                        endTime: period.to,
                    },
                    key: [id, i, period.label, place+"_chart_data"].join('#'),
                    session: broker.guid
                }
            ));
    });
}

function calculatePeriods(id, fromDate, toDate) {
    let toDateObj = moment(toDate);
    let fromDateObj = moment(fromDate);
    let nowObj = moment();

    let delta = moment.duration(toDateObj.diff(fromDate));
    //TODO формат зависит от локали
    let quant = moment.duration(1, 'days');
    let format = 'D MMM';
    if (delta.asDays() < 2) {
        quant = moment.duration(1, 'hours');
        format = 'H ч. D MMM';
        if (delta.asDays() <= 1) {
            format = 'H ч.';
        }
    } else if (delta.asDays() <= 31) {
        // оставим параметры по умолчанию и выйдем
    } else if (delta.asDays() < 70) {
        quant = moment.duration(3, 'days');
    } else if (delta.asWeeks() < 52) {
        quant = moment.duration(1, 'weeks');
    } else {
        quant = moment.duration(1, 'months');
        format = "MMM'YY";
    }
    let periods = [];
    if (quant.asDays() < 1 && toDateObj.isAfter(nowObj)) {
        toDateObj = nowObj.endOf('hour');
    } else {
        toDateObj = toDateObj.endOf('day');
    }
    do {
        let label = toDateObj.format(format);
        chartData.labels.push(label);
        periods.push({to: toDateObj.format(), label: label, from: toDateObj.subtract(quant).format()});  // from.add() меняет сам объект
    } while (toDateObj.isAfter(fromDateObj))
    chartData.fromDate = quant.asDays() != 1 && fromDateObj.format(format);
    // С вычитанием последнего кванта мы можем перевалить за границу интервала, поэтому обрежем последний период
    if (chartData.fromDate) {
        periods[periods.length-1].from = fromDateObj.format();
    }
    periods.reverse();
    chartData.labels.reverse();
    return periods
}

function drawConsumptionChart(id, fromDate) {
    let chartName = 'chart-' + id;
    // при изменении периода удалим старый график, чтоб на его месте построить новый
    $("#" + chartName)?.remove();
    // Create Chart Object
    let chartObj = new Dopyo(chartName, 'Line');
    // данные агрегируются по квантам в точке конечной даты кванта
    // перед созданием графика добавим начало периода на оси со значениями 0, для наглядности
    let addPoint = chartData.fromDate && chartData.fromDate != chartData.labels[0];
    if (addPoint) {
        chartData.labels.splice(0, 0, chartData.fromDate);
    }
    formatLabels();
    chartObj.setLabels(chartData.labels);
    // Set legends and values
    for (let key in chartData[id].legends) {
        let legend = chartData[id].legends[key];
        clearEmptyData(legend.values);
        // добавим 0 для точки начала периода
        if (addPoint) {
            legend.values.splice(0, 0, 0);
        }
        chartObj.addLegend(legend);
    }
    formatChart(chartObj);
    // Inject chart into DOM object
    let div = document.getElementById('consumption-chart-' + id);

    // Draw
    if (chartData[id]?.legends.awada.values.length > 0) {
        chartObj.inject(div);
        chartObj.draw(false);
    } else {
        // удаляем весь <div> с графиком
        deleteChartContainer(div);
        return;
    }
    // Считаем и отображаем экономию
    let cons_elem = document.getElementById("consumption-" + id);
    let cons = document
        .querySelector('[total-power-' + id + ']')
        .textContent;

    cons_elem.textContent = cons + ' ' + gettext('consumption_energy_unit');
    document.getElementById('economy-' + id).textContent = calculateSaving(id) + ' %';
    delete chartData[String(id)];
}

function processChartData(key, data) {
    let [id, index, label, place] = key.split('#');
    // вычислили число периодов, столько точек будем ждать из брокера
    chartData[id].valueCount = chartData[id]?.valueCount || chartData.periods.length * (chartData[id]?.partCount || 1);

    if (Object.keys(data).length === 0) {
        // нет данных, точку не обрабатываем
        if (chartData[id].valueCount) {
            --chartData[id].valueCount;
        }
        return;
    }
    if (index % 2 != 0) {
        label = '';
    }
    if (!chartData.labels[index]) {
        chartData.labels[index] = label;
    }

    let newValue = parseFloat(chartData[id].legends.awada.values[index] || 0.0) + (data.value/1000 || 0.0);
    chartData[id].legends.awada.values[index] = +newValue.toFixed(3);
    chartData[id].legends.withoutAwada.values[index] = consumptionWithoutAwada(id, index, data, place.includes('area'));

    --chartData[id].valueCount;
    if (chartData[id].valueCount <= 0) {
        drawConsumptionChart(id);
    }
}

function clearEmptyData(list) {
    for (let i=0; i<list.length; i++) {
        if (list[i] === undefined) {
            list[i] = 0;
        }
    }
    return list;
}

function consumptionWithoutAwada(id, index, data, isArea = false) {
    let totalPower = +document
        .getElementById(id)
        .getAttribute("total-power-" + id);
    if (!totalPower) {
        return 0.0;
    }
    let hours = ((data.workTime || 0.0) + (data.idleTime || 0.0));
    // для световой зоны определяем, является ли текущее значение максимальным из всех потребителей
    if (isArea) {
        hours = chartData[id].hoursWithoutAwada[index] = Math.max(hours, chartData[id].hoursWithoutAwada[index] || 0);
    }
    return +(totalPower * hours / (60*60*1000*1000)).toFixed(3);
}

function resetChartData(id=null, partCount=1) {
    if (id) {
        chartData[id] = {
            valueCount: 0,  // число число значений, ожидаемых для построения графика
            partCount: partCount,  // число слагаемых в каждой точке (напр. светильников в световой зоне)
            legends: {
                withoutAwada: {"name": "Без AWADA", "stroke": "#B1B9FF", "fill": "#fff", "values": []},
                awada: {"name": "С AWADA", "stroke": "#A700FF", "fill": "#fff", "values": []}
            },
            hoursWithoutAwada: []  // массив для хранения максимального из времен работы потребителей в световой зоне
        };
    } else {
        chartData = {
            labels: [],
            periods: null,
        };
    }

}

function calculateSaving(id) {
    const noAwadaValues = chartData[String(id)].legends.withoutAwada.values;
    const noAwadaConsumption = noAwadaValues.reduce(function (total, item) {
        return total + item;
    }, 0);
    if (noAwadaConsumption == 0) {
        return 0;
    }
    let totalPower = +document
        .getElementById(id)
        .textContent;
    return ((1 - (+totalPower / noAwadaConsumption))*100).toFixed(1);
}

function formatChart(chartObject) {
    chartObject.setLineStyle('straight');
    chartObject.setPadding(50, 20, 50, 10);
    chartObject.setTitleShow(false);
    chartObject.setLegendTableShow(false);
    chartObject.setTooltipShow(false);
    // chartObject.setValueOnChartShow(true);
    chartObject.setGridXShow(true);
    chartObject.setGridYShow(true);
    chartObject.setAnimateShow(false);
}

function deleteChartContainer(id) {
    $('#chart-container-' + id).remove();
}

function formatLabels() {
    let len = chartData.labels.length
    if (len > 16 && len % 3 == 1) {
        for (let i=1; i<len; i++) {
            if (i % 3 != 0) {
                chartData.labels.splice(i, 1, '');
            }
        }
    } else if (len > 14) {
        for (let i=1; i<len; i+=2) {
            chartData.labels.splice(i, 1, '');
        }
    }
}